<?php

if ( ! function_exists( 'wp_tm_hex_to_rgb' ) ) {
	/**
	 * Convert hex color to rgb color
	 *
	 * @param $hex
	 *
	 * @return array|null
	 */
	function wp_tm_hex_to_rgb( $hex ) {
		return sscanf( $hex, "#%02x%02x%02x" );
	}
}

if ( ! function_exists( 'wp_tm_rgb_to_hex' ) ) {
	/**
	 * Convert rgb value to hex color
	 *
	 * @param $r
	 * @param $g
	 * @param $b
	 *
	 * @return string
	 */
	function wp_tm_rgb_to_hex( $r, $g, $b ) {
		return sprintf( "#%02x%02x%02x", $r, $g, $b );
	}
}

if ( ! function_exists( 'wp_tm_array_replace' ) ) {
	/**
	 * Find and replace an item in array
	 *
	 * @param $arr
	 * @param $find
	 * @param $replacement
	 *
	 * @return array
	 */
	function wp_tm_array_replace( $arr, $find, $replacement ) {
		return array_map( function ( $item ) use ( $find, $replacement ) {
			if ( $item === $find ) {
				return $replacement;
			}

			return $item;
		}, $arr );
	}
}

if ( ! function_exists( 'wp_tm_get_prop' ) ) {

	/**
	 * Get a specific property of an array without needing to check if that property exists.
	 */
	function wp_tm_get_prop( $array, $prop, $default = null ) {

		if ( ! is_array( $array ) && ! ( is_object( $array ) && $array instanceof ArrayAccess ) ) {
			return $default;
		}

		if ( isset( $array[ $prop ] ) ) {
			$value = $array[ $prop ];
		} else {
			$value = '';
		}

		return empty( $value ) && null !== $default ? $default : $value;
	}
}

if ( ! function_exists( 'wp_tm_customizer_url' ) ) {
	/**
	 * Get customizer url with autofocus
	 *
	 * @param $type
	 * @param $id
	 *
	 * @return string
	 */
	function wp_tm_customizer_url( $type, $id ) {
		$query                               = array();
		$query[ 'autofocus[' . $type . ']' ] = $id;

		return add_query_arg( $query, admin_url( 'customize.php' ) );
	}
}

if ( ! function_exists( 'wp_tm_array_pluck' ) ) {
	/**
	 * Just like array_pluck function in laravel
	 *
	 * @param $key
	 * @param $arr
	 *
	 * @return array
	 */
	function wp_tm_array_pluck( $key, $arr ) {
		return array_map( function ( $item ) use ( $key ) {
			return $item[ $key ];
		}, $arr );
	}
}

if ( ! function_exists( 'wp_tm_parse_css' ) ) {
	/**
	 * Parse CSS
	 *
	 * @param array $css_output Array of CSS.
	 * @param string $min_media Min Media breakpoint.
	 * @param string $max_media Max Media breakpoint.
	 *
	 * @return string             Generated CSS.
	 */
	function wp_tm_parse_css( $css_output = array(), $min_media = '', $max_media = '' ) {

		$parse_css = '';
		if ( is_array( $css_output ) && count( $css_output ) > 0 ) {

			foreach ( $css_output as $selector => $properties ) {

				if ( null === $properties ) {
					break;
				}

				if ( ! count( $properties ) ) {
					continue;
				}

				$temp_parse_css   = $selector . '{';
				$properties_added = 0;

				foreach ( $properties as $property => $value ) {

					if ( '' === $value ) {
						continue;
					}

					$properties_added ++;
					$temp_parse_css .= $property . ':' . $value . ';';
				}

				$temp_parse_css .= '}';

				if ( $properties_added > 0 ) {
					$parse_css .= $temp_parse_css;
				}
			}

			if ( '' != $parse_css && ( '' !== $min_media || '' !== $max_media ) ) {

				$media_css       = '@media ';
				$min_media_css   = '';
				$max_media_css   = '';
				$media_separator = '';

				if ( '' !== $min_media ) {
					$min_media_css = '(min-width:' . $min_media . 'px)';
				}
				if ( '' !== $max_media ) {
					$max_media_css = '(max-width:' . $max_media . 'px)';
				}
				if ( '' !== $min_media && '' !== $max_media ) {
					$media_separator = ' and ';
				}

				$media_css .= $min_media_css . $media_separator . $max_media_css . '{' . $parse_css . '}';

				return $media_css;
			}
		}

		return $parse_css;
	}
}

if ( ! function_exists( 'wp_tm_get_theme_mod' ) ) {
	/**
	 * Backward compatibility
	 *
	 * @param $name
	 * @param false $default
	 *
	 * @return false|mixed|void
	 */
	function wp_tm_get_theme_mod( $name, $default = false ) {
		return get_option( $name, $default );
	}
}

if ( ! function_exists( 'wp_tm_is_plugin_installed' ) ) {
	/**
	 * Check whether a plugin is installed
	 *
	 * @param $slug
	 *
	 * @return bool
	 */
	function wp_tm_is_plugin_installed( $slug ) {
		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		$all_plugins = get_plugins();

		if ( ! empty( $all_plugins[ $slug ] ) ) {
			return true;
		} else {
			return false;
		}
	}
}

if ( ! function_exists( 'wp_tm_install_plugin' ) ) {
	/**
	 * Install new plugin
	 *
	 * @param $plugin_zip
	 *
	 * @return array|bool|WP_Error
	 */
	function wp_tm_install_plugin( $plugin_zip ) {
		include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		wp_cache_flush();

		$upgrader  = new Plugin_Upgrader();
		$installed = $upgrader->install( $plugin_zip );

		return $installed;
	}
}

if ( ! function_exists( 'wp_tm_upgrade_plugin' ) ) {
	/**
	 * Upgrade plugin
	 *
	 * @param $slug
	 *
	 * @return array|bool|WP_Error
	 */
	function wp_tm_upgrade_plugin( $slug ) {
		include_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		wp_cache_flush();

		$upgrader = new Plugin_Upgrader();
		$upgraded = $upgrader->upgrade( $slug );

		return $upgraded;
	}
}

if ( ! function_exists( 'wp_tm_do_install_plugins' ) ) {
	/**
	 * Install and active plugin
	 *
	 * @param $plugins
	 */
	function wp_tm_do_install_plugins( $plugins ) {
		$back_url = admin_url( 'themes.php?page=wp-tm-theme' );

		echo '<div class="notice notice-warning"><p>';
		echo esc_html__( 'The installation process is starting. This process may take a while on some hosts, so please be patient.', 'temp-mail-x' );
		echo '</p></div>';

		foreach ( $plugins as $slug => $name ) {
			$name        = '<strong>' . $name . '</strong>';
			$plugin_slug = "{$slug}/{$slug}.php";
			$plugin_zip  = "https://downloads.wordpress.org/plugin/{$slug}.latest-stable.zip";

			if ( wp_tm_is_plugin_installed( $plugin_slug ) ) {
				echo '<h4>';
				echo sprintf( esc_html__( 'Upgrading %s ...', 'temp-mail-x' ), $name );
				echo '</h4>';
				wp_tm_upgrade_plugin( $plugin_slug );
				$installed = true;
			} else {
				echo '<h4>';
				echo sprintf( esc_html__( 'Installing %s ...', 'temp-mail-x' ), $name );
				echo '</h4>';
				$installed = wp_tm_install_plugin( $plugin_zip );
			}

			if ( ! is_wp_error( $installed ) && $installed ) {
				echo '<h4>';
				echo sprintf( esc_html__( 'Activating %s ...', 'temp-mail-x' ), $name );
				echo '</h4>';
				activate_plugin( $plugin_slug );
				echo '<div class="updated"><p>';
				echo sprintf( esc_html__( '%s installed successfully', 'temp-mail-x' ), $name );
				echo '</p></div>';
			} else {
				echo '<div class="error"><p>';
				echo sprintf( esc_html__( 'Could not install the %s plugin.', 'temp-mail-x' ), $name );
				echo '</p></div>';
			}

			echo '<br><br>';
		}

		echo '<p>';
		esc_html_e( 'All Done!', 'temp-mail-x' );
		echo '</p>';
		echo '<p><a href="' . esc_url( $back_url ) . '">' . esc_html__( 'Return To Theme Page', 'temp-mail-x' ) . '</a></p>';
	}
}
